# TPK4186 - Advanced Tools for Performance Engineering
# XLSInterface.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required modules
# 2. XLS Printer
# 3. XLS Parser

# 1. Required modules
# -------------------

from Retailer import *
import openpyxl

# 2. XLS Printer
# --------------

class XLSPrinter:

  def ExportRetailerSalesAtXLSFormat(self, retailer, fileName):
    workbook = openpyxl.Workbook()
    worksheet = workbook.active
    worksheet.title = "Sales"
    self.UploadRetailerSalesInWorksheet(retailer, worksheet)
    try:
      workbook.save(fileName)
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    return 0

  def UploadRetailerSalesInWorksheet(self, retailer, worksheet):
    worksheet['A1'] = "Product"
    worksheet['B1'] = "Profit"
    worksheet['C1'] = "Jan"
    worksheet['D1'] = "Feb"
    worksheet['E1'] = "Mar"
    worksheet['F1'] = "Apr"
    worksheet['G1'] = "May"
    worksheet['H1'] = "Jun"
    worksheet['I1'] = "Jul"
    worksheet['J1'] = "Aug"
    worksheet['K1'] = "Sep"
    worksheet['L1'] = "Oct"
    worksheet['M1'] = "Nov"
    worksheet['N1'] = "Dec"
    row = 1
    for product in retailer.GetProducts():
      row += 1
      self.UploadProductSalesInWorksheet(product, row, worksheet)

  def UploadProductSalesInWorksheet(self, product, row, worksheet):
    worksheet.cell(row=row, column=1, value=product.GetName())
    self.UploadCurrency(product.GetProfitPerSale(), row, 2, worksheet)
    for monthNumber in range(1, 13):
      worksheet.cell(row=row, column=monthNumber+2, value=product.GetMonthlySales(monthNumber))

  def UploadCurrency(self, value, row, column, worksheet):
    worksheet.cell(row=row, column=column, value="{0:.2f}".format(value))

  def ExportRetailerSalesAndProfitAtXLSFormat(self, retailer, fileName):
    workbook = openpyxl.Workbook()
    worksheetSales = workbook.active
    worksheetSales.title = "Sales"
    worksheetProfit = workbook.create_sheet("Profits")
    self.UploadRetailerSalesInWorksheet(retailer, worksheetSales)
    self.UploadRetailerProfitInWorksheet(retailer, worksheetProfit)
    try:
      workbook.save(fileName)
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    return 0

  def UploadRetailerProfitInWorksheet(self, retailer, worksheet):
    worksheet['A1'] = "Product"
    worksheet['B1'] = "Jan"
    worksheet['C1'] = "Feb"
    worksheet['D1'] = "Mar"
    worksheet['E1'] = "Apr"
    worksheet['F1'] = "May"
    worksheet['G1'] = "Jun"
    worksheet['H1'] = "Jul"
    worksheet['I1'] = "Aug"
    worksheet['J1'] = "Sep"
    worksheet['K1'] = "Oct"
    worksheet['L1'] = "Nov"
    worksheet['M1'] = "Dec"
    row = 1
    for product in retailer.GetProducts():
      row += 1
      self.UploadProductProfitInWorksheet(product, row, worksheet)
    row += 1
    worksheet.cell(row=row, column=1, value="Total")
    for monthNumber in range(1, 13):
      column = chr(ord("A") + monthNumber)
      formula = "=SUM(%s2:%s%d)" % (column, column, row-1)
      worksheet.cell(row=row, column=monthNumber+1, value=formula)
    formula = "=SUM(N2:N%d)" % (row-1)
    worksheet.cell(row=row, column=14, value=formula)

  def UploadProductProfitInWorksheet(self, product, row, worksheet):
    worksheet.cell(row=row, column=1, value=product.GetName())
    for monthNumber in range(1, 13):
      column = chr(ord("B") + monthNumber)
      formula = "=Sales!B%d*Sales!%s%d" % (row, column, row)
      worksheet.cell(row=row, column=monthNumber+1, value=formula)
    formula = "=SUM(B%d:M%d)" % (row, row)
    worksheet.cell(row=row, column=14, value=formula)

# 3. XLS Parser
# -------------

class XLSParser:

  def ImportRetailerSalesAtXLSFormat(self, retailer, fileName):
    try:
      workbook = openpyxl.load_workbook(fileName)
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    worksheet = workbook.active
    error = self.DownloadRetailerSalesAtXLSFormat(retailer, worksheet)
    return error

  def DownloadRetailerSalesAtXLSFormat(self, retailer, worksheet):
    row = 1
    while True:
      row += 1
      cell = worksheet.cell(row=row, column=1)
      productName = cell.value
      if productName==None:
        break
      product = retailer.NewProduct(productName)
      cell = worksheet.cell(row=row, column=2)
      profitPerSale = cell.value
      if profitPerSale==None:
        sys.stderr.write('Row %d, column 2:  not a correct profit per sale"\n' % lineNumber)
        sys.stderr.flush()
        return 1
      product.SetProfitPerSale(float(profitPerSale))
      for monthNumber in range(1, 13):
        cell = worksheet.cell(row=row, column=monthNumber+2)
        quantity = cell.value
        if quantity==None:
          sys.stderr.write('Line %d, column %d: not a correct monthly quantity"\n' % (lineNumber, monthNumber+2))
          sys.stderr.flush()
          return 1
        product.SetMonthlySales(monthNumber, quantity)
    return 0


